@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: CurrentWindowCmd.cc,v 1.9 2004/02/20 19:40:31 fluxgen Exp $
+// $Id: CurrentWindowCmd.cc 3731 2004-11-30 01:23:24Z mathias $
 
 #include "CurrentWindowCmd.hh"
 
@@ -86,14 +86,17 @@ void MoveCmd::real_execute() {
 }
 
 ResizeCmd::ResizeCmd(const int step_size_x, const int step_size_y) :
-  m_step_size_x(step_size_x), m_step_size_y(step_size_y) { }
+    m_step_size_x(step_size_x), m_step_size_y(step_size_y) { }
 
 void ResizeCmd::real_execute() {
-  fbwindow().resize(
-                    fbwindow().width() + 
-                    m_step_size_x * fbwindow().winClient().width_inc,
-                    fbwindow().height() + 
-                    m_step_size_y * fbwindow().winClient().height_inc);
+  
+    int w = std::max<int>(static_cast<int>(fbwindow().width() + 
+                                      m_step_size_x * fbwindow().winClient().width_inc), 
+                     fbwindow().frame().titlebarHeight() * 2 + 10);
+    int h = std::max<int>(static_cast<int>(fbwindow().height() + 
+                                      m_step_size_y * fbwindow().winClient().height_inc), 
+                     fbwindow().frame().titlebarHeight() + 10);
+    fbwindow().resize(w, h);
 }
 
 MoveToCmd::MoveToCmd(const int step_size_x, const int step_size_y) :
@@ -107,5 +110,6 @@ ResizeToCmd::ResizeToCmd(const int step_size_x, const int step_size_y) :
     m_step_size_x(step_size_x), m_step_size_y(step_size_y) { }
 
 void ResizeToCmd::real_execute() {
-    fbwindow().resize(m_step_size_x, m_step_size_y);
+    if (m_step_size_x > 0 && m_step_size_y > 0)
+        fbwindow().resize(m_step_size_x, m_step_size_y);
 }
