@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbCommandFactory.cc,v 1.29 2004/04/22 21:12:34 fluxgen Exp $
+// $Id: FbCommandFactory.cc 3732 2004-11-30 01:26:00Z mathias $
 
 #include "FbCommandFactory.hh"
 
@@ -66,13 +66,20 @@ FbCommandFactory::FbCommandFactory() {
         "commanddialog",
         "deiconify",
         "detachclient",
+        "export",
         "exec",
         "execcommand",
         "execute",
+        "exit",
+        "focusup",
+        "focusdown",
+        "focusleft",
+        "focusright",
         "iconify",
         "killwindow",
         "leftworkspace",
         "lower",
+        "lowerlayer",
         "macrocmd",
         "maximize",
         "maximizehorizontal",
@@ -98,6 +105,8 @@ FbCommandFactory::FbCommandFactory() {
         "prevworkspace",
         "quit",
         "raise",
+        "raiselayer",
+        "reconfig",
         "reconfigure",
         "reloadstyle",
         "resizeto",
@@ -108,6 +117,7 @@ FbCommandFactory::FbCommandFactory() {
         "rightworkspace",
         "rootmenu",
         "saverc",
+        "setenv",
         "sendtoworkspace",
         "sendtonextworkspace",
         "sendtoprevworkspace",
@@ -124,6 +134,20 @@ FbCommandFactory::FbCommandFactory() {
         "toggledecor",
         "windowmenu",
         "workspace",
+        /* NOTE: The following are DEPRECATED and subject to removal */
+        "workspace1",
+        "workspace2",
+        "workspace3",
+        "workspace4",
+        "workspace5",
+        "workspace6",
+        "workspace7",
+        "workspace8",
+        "workspace9",
+        "workspace10",
+        "workspace11",
+        "workspace12",
+        /* end note */
         "workspacemenu",
         ""
     };
@@ -144,7 +168,7 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
     //
     if (command == "restart")
         return new RestartFluxboxCmd(arguments);
-    else if (command == "reconfigure")
+    else if (command == "reconfigure" || command == "reconfig")
         return new ReconfigureFluxboxCmd();
     else if (command == "setstyle")
         return new SetStyleCmd(arguments);
@@ -154,6 +178,21 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new SaveResources();
     else if (command == "execcommand" || command == "execute" || command == "exec")
         return new ExecuteCmd(arguments); // execute command on key screen
+    else if (command == "exit")
+        return new ExitFluxboxCmd();
+    else if (command == "setenv" || command == "export") {
+
+        std::string name = arguments;
+        FbTk::StringUtil::removeFirstWhitespace(name);
+        FbTk::StringUtil::removeTrailingWhitespace(name);
+        size_t pos = name.find_first_of(command == "setenv" ? " \t" : "=");
+        if (pos == std::string::npos || pos == name.size())
+            return 0;
+        
+        std::string value = name.substr(pos + 1);
+        name = name.substr(0, pos);
+        return new ExportCmd(name, value);
+    }
     else if (command == "quit")
         return new FbTk::SimpleCommand<Fluxbox>(*Fluxbox::instance(), &Fluxbox::shutdown);
     else if (command == "commanddialog") // run specified fluxbox command
@@ -173,7 +212,6 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         // i.e so users can set space before workspace name and so on
         string value = name.substr(pos + 1);
         name = name.substr(0, pos);
-        cerr<<"Creating SetResourceValue "<<name<<" "<<value<<endl;
         return new SetResourceValueCmd(name, value);
     } else if (command == "setresourcevaluedialog")
         return new SetResourceValueDialogCmd();
@@ -226,8 +264,12 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new MoveCmd(0,atoi(arguments.c_str()));
     else if (command == "raise")
         return new CurrentWindowCmd(&FluxboxWindow::raise);
+    else if (command == "raiselayer")
+        return new CurrentWindowCmd(&FluxboxWindow::raiseLayer);
     else if (command == "lower")
         return new CurrentWindowCmd(&FluxboxWindow::lower);
+    else if (command == "lowerlayer")
+        return new CurrentWindowCmd(&FluxboxWindow::lowerLayer);
     else if (command == "close")
         return new CurrentWindowCmd(&FluxboxWindow::close);
     else if (command == "shade" || command == "shadewindow")
@@ -242,7 +284,7 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new SendToNextWorkspaceCmd(atoi(arguments.c_str()));
     else if (command == "sendtoprevworkspace")
         return new SendToPrevWorkspaceCmd(atoi(arguments.c_str()));
-    else if (command == "killwindow")
+    else if (command == "killwindow" || command == "kill")
         return new KillWindowCmd();
      else if (command == "nexttab")
         return new CurrentWindowCmd(&FluxboxWindow::nextClient);
@@ -272,10 +314,24 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         if (!arguments.empty())
             num = atoi(arguments.c_str());
         return new JumpToWorkspaceCmd(num-1);
+    } if (command.substr(0, 9) == "workspace" && command[9] >= '0' && command[9] <= '9') {
+        cerr<<"*** WARNING: 'Workspace<n>' actions are deprecated! Use 'Workspace <n>' instead"<<endl;
+        int num = 1;
+        num = atoi(command.substr(9).c_str());
+        return new JumpToWorkspaceCmd(num-1);
+        
     } else if (command == "nextwindow")
         return new NextWindowCmd(atoi(arguments.c_str()));
     else if (command == "prevwindow")
         return new PrevWindowCmd(atoi(arguments.c_str()));
+    else if (command == "focusup")
+        return new DirFocusCmd(BScreen::FOCUSUP);
+    else if (command == "focusdown")
+        return new DirFocusCmd(BScreen::FOCUSDOWN);
+    else if (command == "focusleft")
+        return new DirFocusCmd(BScreen::FOCUSLEFT);
+    else if (command == "focusright")
+        return new DirFocusCmd(BScreen::FOCUSRIGHT);
     else if (command == "nextgroup")
         return new NextWindowCmd(atoi(arguments.c_str()) ^ BScreen::CYCLEGROUPS);
     else if (command == "prevgroup")
