@@ -22,13 +22,14 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc,v 1.12 2004/01/21 14:15:29 fluxgen Exp $
+// $Id: ImageControl.cc 3700 2004-11-19 11:37:27Z mathias $
 
 #include "ImageControl.hh"
 
 #include "TextureRender.hh"
 #include "App.hh"
 #include "SimpleCommand.hh"
+#include "I18n.hh"
 
 //use GNU extensions
 #ifndef _GNU_SOURCE
@@ -44,9 +45,21 @@
 #include <sys/types.h>
 #endif // HAVE_SYS_TYPES_H
 
-#include <cstdlib>
-#include <cstring>
-#include <cstdio>
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
+#else
+  #include <stdio.h>
+#endif
 
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>
@@ -107,7 +120,7 @@ ImageControl::ImageControl(int screen_num, bool dither,
         m_timer.setCommand(clean_cache);
         m_timer.start();
     }
-	
+
     createColorTable();
 }
 
@@ -126,13 +139,15 @@ ImageControl::~ImageControl() {
         delete [] grad_ybuffer;
     }
 
+    Display *disp = FbTk::App::instance()->display();
+
     if (m_colors) {
         unsigned long *pixels = new unsigned long [m_num_colors];
 
         for (unsigned int color = 0; color < m_num_colors; color++)
             *(pixels + color) = (*(m_colors + color)).pixel;
 
-        XFreeColors(FbTk::App::instance()->display(), m_colormap, pixels, m_num_colors, 0);
+        XFreeColors(disp, m_colormap, pixels, m_num_colors, 0);
 
         delete [] m_colors;
     }
@@ -140,7 +155,6 @@ ImageControl::~ImageControl() {
     if (cache.size() > 0) {
         CacheList::iterator it = cache.begin();
         CacheList::iterator it_end = cache.end();
-        Display *disp = FbTk::App::instance()->display();
         for (; it != it_end; ++it) {
             XFreePixmap(disp, (*it)->pixmap);
             delete (*it);
@@ -153,7 +167,7 @@ ImageControl::~ImageControl() {
 
 Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
                                  const Texture &text) const {
-    
+
     if (text.pixmap().drawable() != None) {
         // do comparsion with width/height and texture_pixmap
         CacheList::iterator it = cache.begin();
@@ -177,27 +191,28 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
     tmp.pixel1 = text.color().pixel();
     tmp.pixel2 = text.colorTo().pixel();
     */
-    CacheList::iterator it = cache.begin(); 	 
-    CacheList::iterator it_end = cache.end(); 	 
-    for (; it != it_end; ++it) { 	 
-        if (((*it)->width == width) && 	 
-            ((*it)->height == height) && 	 
-            ((*it)->texture == text.type()) && 	 
-            ((*it)->pixel1 == text.color().pixel())) { 	 
-            if (text.type() & FbTk::Texture::GRADIENT) { 	 
-                if ((*it)->pixel2 == text.colorTo().pixel()) { 	 
-                    (*it)->count++; 	 
-                    return (*it)->pixmap; 	 
-                } 	 
-            } else { 	 
-                (*it)->count++; 	 
-                return (*it)->pixmap; 	 
-            } 	 
-        } 	 
+
+    CacheList::iterator it = cache.begin();
+    CacheList::iterator it_end = cache.end();
+    for (; it != it_end; ++it) {
+        if (((*it)->width == width) &&
+            ((*it)->height == height) &&
+            ((*it)->texture == text.type()) &&
+            ((*it)->pixel1 == text.color().pixel())) {
+            if (text.type() & FbTk::Texture::GRADIENT) {
+                if ((*it)->pixel2 == text.colorTo().pixel()) {
+                    (*it)->count++;
+                    return (*it)->pixmap;
+                }
+            } else {
+                (*it)->count++;
+                return (*it)->pixmap;
+            }
+        }
     }
 
     return None;
-    
+
 }
 
 
@@ -235,7 +250,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
         else
             tmp->pixel2 = 0l;
 
-        cache.push_back(tmp); 
+        cache.push_back(tmp);
 
         if ((unsigned) cache.size() > cache_max)
             cleanCache();
@@ -328,10 +343,10 @@ void ImageControl::getGradientBuffers(unsigned int w,
 
 
 void ImageControl::installRootColormap() {
-    XGrabServer(FbTk::App::instance()->display());
-
 
     Display *disp = FbTk::App::instance()->display();
+    XGrabServer(disp);
+
     bool install = true;
     int i = 0, ncmap = 0;
     Colormap *cmaps =
@@ -342,14 +357,14 @@ void ImageControl::installRootColormap() {
             if (*(cmaps + i) == m_colormap)
                 install = false;
         }
-		
+
         if (install)
             XInstallColormap(disp, m_colormap);
 
         XFree(cmaps);
     }
 
-    XUngrabServer(FbTk::App::instance()->display());
+    XUngrabServer(disp);
 }
 
 
@@ -387,7 +402,7 @@ void ImageControl::cleanCache() {
             deadlist.push_back(it);
             delete tmp;
             tmp=0;
-        } 
+        }
     }
 
     std::list<CacheList::iterator>::iterator dead_it = deadlist.begin();
@@ -395,7 +410,7 @@ void ImageControl::cleanCache() {
     for (; dead_it != dead_it_end; ++dead_it) {
         cache.erase(*dead_it);
     }
-    
+
 }
 
 void ImageControl::createColorTable() {
@@ -496,7 +511,7 @@ void ImageControl::createColorTable() {
                 }
             }
         }
-		
+
         for (unsigned int i = 0; i < m_num_colors; i++) {
             if (! XAllocColor(disp, m_colormap, &m_colors[i])) {
                 fprintf(stderr, "couldn't alloc color %i %i %i\n",
@@ -505,7 +520,7 @@ void ImageControl::createColorTable() {
             } else
                 m_colors[i].flags = DoRed|DoGreen|DoBlue;
         }
-		
+
         XColor icolors[256];
         unsigned int incolors = (((1 << m_screen_depth) > 256) ? 256 : (1 << m_screen_depth));
 
@@ -517,7 +532,7 @@ void ImageControl::createColorTable() {
             if (! m_colors[i].flags) {
                 unsigned long chk = 0xffffffff, pixel, close = 0;
                 char p = 2;
-		
+
                 while (p--) {
                     for (unsigned int ii = 0; ii < incolors; ii++) {
                         int r = (m_colors[i].red - icolors[i].red) >> 8;
@@ -638,7 +653,8 @@ void ImageControl::createColorTable() {
         }
 
     default:
-        cerr<<"FbTk::ImageControl: Unsupported visual"<<endl;
+        _FB_USES_NLS;
+        cerr<<"FbTk::ImageControl: "<<_FBTKTEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X")<<endl;
         break;
     }
 }
